# -*- coding: utf-8 -*-
import json, datetime
try:
    from urllib2 import urlopen, Request
except ImportError:
    from urllib.request import urlopen, Request
try:
    from urllib import quote
except ImportError:
    from urllib.parse import quote
try:
    from urlparse import urlparse, parse_qs, urljoin
except ImportError:
    from urllib.parse import urlparse, parse_qs, urljoin
ua = 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/87.0.4280.88 Safari/537.36'


def getjson(url):
    request = Request(url, headers={'User-Agent': ua})
    return json.loads(urlopen(request).read().decode('utf-8'))


def search(s, t, page=1):
    if not s:
        return {'code': 'bilibili 300'}
    if t == 'video':
        t = 'video'
        k = 'search_video'
    elif t == 'space':
        t = 'bili_user'
        k = 'search_space'
    else:
        return {'code': 'haitum 200', 'error': str(e)}
    url = 'https://api.bilibili.com/x/web-interface/search/type?keyword=%s&search_type=%s&page=%d' % (quote(s), t, page)
    try:
        data = getjson(url)
    except Exception as e:
        return {'code': 'haitum 201', 'error': str(e)}
    ret = []
    for e in data['data']['result']:
        if e['type'] == 'video':
            title = e['title']
            title = title.replace('<em class="keyword">', '')
            title = title.replace('</em>', '')
            img = e['pic']
            url = 'https://www.bilibili.com/video/%s' % e['bvid']
            plot = []
            plot.append(e['author'])
            plot.append(datetime.datetime.utcfromtimestamp(e['pubdate']).strftime('%Y-%m-%d %H:%M:%S'))
            plot.append(e['description'])
        if e['type'] == 'bili_user':
            title = e['uname']
            img = e['upic']
            url = 'https://space.bilibili.com/%d/video' % e['mid']
            plot = []
            plot.append(u'稿件: %d' % e['videos'])
            plot.append(u'粉丝: %d' % e['fans'])
        plot = '\n'.join(plot)
        ret.append({'title': title, 'url': url, 'img': urljoin(url, img), 'plot': plot})
    if page < int(data['data']['numPages']):
        ret.append({'title': 'Next Page (%d)' % (page + 1), k: s, 'page': page + 1})
    return ret


def videolist(url):
    o = urlparse(url)
    p = o.path.split('/')
    q = parse_qs(o.query, True)
    url1 = 'https://api.bilibili.com/x/player/pagelist?bvid=%s' % p[-1]
    if 'p' in q:
        return ykdl(url)
    try:
        data = getjson(url1)
    except Exception as e:
        return {'code': 'haitum 301', 'error': str(e)}
    if len(data['data']) == 1:
        return ykdl(url)
    ret = []
    for e in data['data']:
        ret.append({'title': '%d. %s' % (e['page'], e['part']), 'url': 'https://www.bilibili.com/video/%s?p=%d' % (url, e['page'])})
    return ret


def spacelist(url, page):
    o = urlparse(url)
    p = o.path.split('/')
    q = parse_qs(o.query, True)
    url1 = 'https://api.bilibili.com/x/space/arc/search?mid=%s&pn=%d' % (p[-2], page)
    try:
        data = getjson(url1)
    except Exception as e:
        return {'code': 'haitum 401', 'error': str(e)}
    ret = []
    for e in data['data']['list']['vlist']:
        title = e['title']
        img = e['pic']
        url2 = 'https://www.bilibili.com/video/%s' % e['bvid']
        plot = []
        plot.append(e['author'])
        plot.append(datetime.datetime.utcfromtimestamp(e['created']).strftime('%Y-%m-%d %H:%M:%S'))
        plot.append(e['description'])
        plot = '\n'.join(plot)
        ret.append({'title': title, 'url': url2, 'img': urljoin(url, img), 'plot': plot})
    total_page = (data['data']['page']['count'] + data['data']['page']['ps'] - 1) // data['data']['page']['ps']
    if page < total_page:
        ret.append({'title': 'Next Page (%d / %d)' % (page + 1, total_page), 'url': url, 'page': page + 1})
    return ret


def ykdl(url):
    from ykdlWrapper import url_to_module
    m, u = url_to_module(url)
    parser = m.parser
    parser(u)
    info = m.prepare()
    info = info.jsonlize()
    ret = []
    for k, v in info['streams'].items():
        ret.append({'title': k, 'url': v['src']})
    return ret
